#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import hashlib
import argparse

import warnings
warnings.filterwarnings("ignore")

__progname__ = 'hashr'
__version__  = '0.3'
__header__ = '''
  ___  ____  ____  ____     ____   __  
 / __)(  __)(  _ \(_  _)___(  _ \ / _\ 
( (__  ) _)  )   /  )( (___)) __//    \\
 \___)(____)(__\_) (__)    (__)  \_/\_/ 

  www.cert-pa.it | cert-pa@cert-pa.it
  '''+__progname__+''' v.'''+__version__+'''
________________________________________________________________________
'''

__epilog__ = '''
________________________________________________________________________

EXAMPLE - Search recursively in your system:

  hashr md5 c:\\ -r --hashlist iocfile.txt

'''

__description__ = 'hashr is a tool able to compute hash of the files and compare them with a hashlist file.\n\
Using hashr you can verify if IoC malware hashes (like APT) are present in your system.\n\n'
__license__ = 'GNU GPL 3 License'
__source__ = 'https://www.cert-pa.it'
__author__ = 'CERT-PA'

from modules import checkinhashlist, gethash, hashmap

def validate_algorithm(algorithm):
	algorithm_list = ['md5', 'sha1', 'sha256', 'imphash']
	if algorithm not in algorithm_list:
		print('Hash algorithm not valid')
		sys.exit()

def validate_file(filename):
	if not os.path.isfile(filename):
		return False
	else:
		return True

def validate_folder(path):
	if not os.path.isdir(path):
		return False
	else:
		return True

def validate_filetype(filetype):
	if filetype and not target:
		print('Use --filetype on directory target')
		sys.exit()

	if filetype and not exclude:
		if filetype.find(',') == -1:
			filetype = (filetype,)
		else:
			filetype = tuple(item for item in filetype.split(',') if item.strip())
	else:
		if filetype and exclude:
			if filetype.find(',') == -1:
				filetype = (filetype,)
			else:
				filetype = tuple(item for item in filetype.split(',') if item.strip())

	return filetype

def writefile(filename, content):
	out_file = open(filename,'a')
	out_file.write(content+'\n')
	out_file.close()

print(__header__)

parser = argparse.ArgumentParser(
			version=__version__,
			formatter_class=argparse.RawTextHelpFormatter,
			prog=__progname__,
			description=__description__,
			epilog = __epilog__)
		
parser.add_argument('HASH', help='algorithm supported: md5, sha1, sha256, imphash')
parser.add_argument('TARGET', help='file or directory name from which to obtain the hash')
parser.add_argument('-r', '--recursive', help='recursive directory', action='store_true')
parser.add_argument('-d', '--duplicate', help='show duplicate hashes found', action='store_true')
parser.add_argument('--filetype', dest='filetype', help='filter for extension (use comma separator)')
parser.add_argument('-e', '--exclude', help='exclude filetype', action='store_true')
parser.add_argument('--hashlist', dest='file', help='load file with homogeneous hashes list')
parser.add_argument('--encrypted', help='only for encrypted hashlist', action='store_true')
parser.add_argument('-o', dest='output', help='write output file')
args = parser.parse_args()

'''
# debug
args_dict = vars(parser.parse_args())
print(args_dict)
'''

algorithm = args.HASH
target = args.TARGET
hashlist = args.file
recursive = args.recursive
duplicate = args.duplicate
filetype = args.filetype
exclude = args.exclude
encrypted = args.encrypted
output = args.output

validate_algorithm(algorithm)
if not validate_folder(target) and not validate_file(target):
	print('File or folder not found')
	sys.exit()
filetype = validate_filetype(filetype)

if hashlist and validate_file(hashlist):
	result = checkinhashlist.hashfolder(algorithm, target, hashlist, recursive, duplicate, filetype, exclude, encrypted)
	if result and output:
		for res in result:
			writefile(output,' '.join(res))
	sys.exit()

elif validate_folder(target) and not hashlist:
	hashmap.hashfolder(algorithm, target, recursive, duplicate, filetype, exclude)
	sys.exit()

elif validate_file(target) and hashlist and validate_file(hashlist) and not recursive and not duplicate:
	checkinhashlist.hashfile(algorithm, target, hashlist, encrypted)
	sys.exit()

elif not hashlist and not recursive and not duplicate:
	print gethash.filename(algorithm, target)
	sys.exit()

else:
	exit('Syntax error in arguments. Use -h option to help')

