import sys
import time
import hashlib

import hashcalc
import filepath
import hashlistload

# empty file hash
empty = [
	'd41d8cd98f00b204e9800998ecf8427e',
	'da39a3ee5e6b4b0d3255bfef95601890afd80709',
	'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
]


def get_hashlist(digest, path_to_hashlist, encrypted):
	if encrypted: 
		hashlist = hashlistload.fromfile('md5', path_to_hashlist, encrypted)
	else:
		hashlist = hashlistload.fromfile(digest, path_to_hashlist)

	if not hashlist: 
		print('hashlist file is empty '+str(path_to_hashlist))
		sys.exit()

	return hashlist


def scan(digest, file, encrypted):
	if digest == 'md5':	
		hashval = hashcalc.md5(file)
		if encrypted: 
			hashval = hashlib.md5(str(hashval))
			hashval = hashval.hexdigest()

	elif digest == 'sha1': 
		hashval = hashcalc.sha1(file)
		if encrypted: 
			hashval = hashlib.md5(str(hashval))
			hashval = hashval.hexdigest()
		
	elif digest == 'sha256': 
		hashval = hashcalc.sha256(file)
		if encrypted: 
			hashval = hashlib.md5(str(hashval))
			hashval = hashval.hexdigest()
		
	elif digest == 'imphash': 
		hashval = hashcalc.imphash(file)
		if encrypted: 
			hashval = hashlib.md5(str(hashval))
			hashval = hashval.hexdigest()

	else:
		print('Hash algorithm not valid')
		sys.exit()

	return hashval


def hashfile(digest, path_to_fname, path_to_hashlist, encrypted):
	hashlist = get_hashlist(digest, path_to_hashlist, encrypted)
	
	hashlist_count = len(hashlist)
	print('--[ hashlist encrypted : '+str(encrypted))
	print('--[ hashlist path      : '+str(path_to_hashlist))
	print('--[ hashes loaded      : '+str(hashlist_count))
	print('--[ file ready         : '+str(path_to_fname))
	print('--[ searching for      : '+str(digest)+'\n')

	filecount = 1
	processed = 0
	match = 0
	
	start_time = time.time()

	file = path_to_fname
	hashval = scan(digest, file, encrypted)

	if hashval in empty: 
		empty_list.append([hashval, file])

	if hashval != None:
		processed += 1
		if hashval in hashlist:
			match += 1
			print(hashval)

	print('\n--[ '+str(processed)+' files processed in %s seconds' % (time.time() - start_time))
	print('--[ '+str(match)+' file found in hashlist')
	print('--[ '+str(len(empty_list))+' empty file found')


def hashfolder(digest, path_to_folder, path_to_hashlist, recursive, duplicate, filetype, exclude, encrypted):
	
	hashlist = get_hashlist(digest, path_to_hashlist, encrypted)
	hashlist_count = len(hashlist)

	print('--[ directory mapping  : '+str(path_to_folder))
	if recursive: print('--[ recursive mode     : '+str(recursive))
	if duplicate: print('--[ duplicate check    : '+str(duplicate))
	if filetype:  print('--[ filetype extension : '+str(filetype))
	if exclude:   print('--[ exclude mode       : '+str(exclude))
	print('--[ hashlist encrypted : '+str(encrypted))
	print('--[ hashlist path      : '+str(path_to_hashlist))
	print('--[ hashes loaded      : '+str(hashlist_count))
	
	filelist = filepath.getlist(path_to_folder, recursive, filetype, exclude)
	filecount = len(filelist)

	print('--[ files ready        : '+str(filecount))
	print('--[ searching for      : '+str(digest)+'\n')

	processed = 0
	match = 0
	hashs_list = []
	empty_list = []
	start_time = time.time()

	for file in filelist:

		hashval = scan(digest, file, encrypted)
		
		if hashval in empty: 
			empty_list.append([hashval, file])

		if hashval != None:
			processed += 1
			if hashval in hashlist:
				match += 1
				print (hashval, file)
				hashs_list.append([hashval, file])

	print('\n--[ '+str(processed)+' files processed in %s seconds' % (time.time() - start_time))
	print('--[ '+str(match)+' file found in hashlist')
	
	hashlist = []
	''' formatting original list '''
	for item in hashs_list:
		hashlist.append(item[0])
	
	hashduple = []
	
	if duplicate:
		for i in set(hashlist):
			value = hashlist.count(i)
			if value > 1: hashduple.append(i+' '+str(value))
	
		if hashduple:
			print('\r')
			for duple in hashduple:
				print(duple)
			print('\r')

		print('--[ '+str(len(hashduple))+' duplicate hash found')
	
	else:
		print('--[ '+str(len(hashlist)-len(set(hashlist)))+' duplicate hash found')
	
	print('--[ '+str(len(empty_list))+' empty file found')

	return hashs_list
